@@ -54,7 +54,7 @@ class ResourceMeta(MethodViewType):
 class Resource(MethodView):
     """Base resource class"""
 
-    def __new__(cls, **kwargs):
+    def __new__(cls, *args, **kwargs):
         """Constructor of a resource instance"""
         if hasattr(cls, '_data_layer'):
             cls._data_layer.resource = cls
@@ -62,21 +62,29 @@ class Resource(MethodView):
         return super(Resource, cls).__new__(cls)
 
     def __init__(self, endpoint=None, request_parsers=None, response_renderers=None):
-        # Start with default parsers, but accept user provided ones
-        self.request_parsers = {
+        # Start with default parsers, then combine them with class variables, and also
+        # constructor arguments
+        combined_parsers = {
             'application/vnd.api+json': parse_json,
             'application/json': parse_json
         }
+        if hasattr(self, 'request_parsers'):
+            combined_parsers.update(self.request_parsers)
         if request_parsers is not None:
-            self.request_parsers.update(request_parsers)
+            combined_parsers.update(request_parsers)
+        self.request_parsers = combined_parsers
 
-        # Start with default renderers, but accept user provided ones
-        self.response_renderers = {
+        # Start with default renderers, then combine them with class variables, and also
+        # constructor arguments
+        combined_renderers = {
             'application/vnd.api+json': render_json,
             'application/json': render_json
         }
+        if hasattr(self, 'response_renderers'):
+            combined_renderers.update(self.response_renderers)
         if response_renderers is not None:
-            self.response_renderers.update(response_renderers)
+            combined_renderers.update(response_renderers)
+        self.response_renderers = combined_renderers
 
         # By default we assign each Resource class with a view/endpoint. However if the
         # same resource is used for multiple routes, the endpoint will be overwritten.
@@ -85,18 +93,44 @@ class Resource(MethodView):
             self.view = endpoint
 
     def parse_request(self):
-        return self.request_parsers[request.content_type](request)
+        return self.request_parsers[request.mimetype](request)
+
+    def _access_kwargs(self, name, args, kwargs):
+        """
+        Gets the kwargs dictionary with the provided name. This can be implemented as
+        a dictionary *or* a function, so we have to handle both possibilities
+        """
+        # Access the field
+        val = getattr(self, name, dict())
+
+        if callable(val):
+            # If it's a function, call it and validate its result
+            schema_kwargs = val(args, kwargs)
+            if not isinstance(schema_kwargs, dict):
+                raise TypeError(
+                    'The return value of the "{}" function must be a dictionary of kwargs'
+                )
+        else:
+            # If it's a dictionary, use it directly
+            schema_kwargs = val
+            if not isinstance(schema_kwargs, dict):
+                raise TypeError(
+                    'The value of the "{}" class variable must be a dictionary of kwargs'
+                )
+
+        return schema_kwargs
 
     @jsonapi_exception_formatter
     def dispatch_request(self, *args, **kwargs):
         """Logic of how to handle a request"""
-        method = getattr(self, request.method.lower(), None)
+        method_name = request.method.lower()
+        method = getattr(self, method_name, None)
         if method is None and request.method == 'HEAD':
             method = getattr(self, 'get', None)
         assert method is not None, 'Unimplemented method {}'.format(request.method)
 
         # Before we defer to the method function, parse the incoming request
-        if request.content_type not in self.request_parsers:
+        if method_name in ('post', 'patch', 'put') and request.mimetype not in self.request_parsers:
             raise InvalidContentType(
                 'This endpoint only supports the following request content types: {}'.format(', '.join(
                     self.request_parsers.keys())
@@ -104,18 +138,18 @@ class Resource(MethodView):
             )
 
         # Choose a renderer based on the Accept header
-        if len(request.accept_mimetypes) < 1:
-            # If the request doesn't specify a mimetype, assume JSON API
+        if request.accept_mimetypes.best in self.response_renderers:
+            accept_type = request.accept_mimetypes.best
+        elif request.accept_mimetypes.accept_json or len(request.accept_mimetypes) < 1:
             accept_type = 'application/vnd.api+json'
-        elif request.accept_mimetypes.best not in self.response_renderers:
+        else:
             # Check if we support the response type
             raise InvalidAcceptType(
                 'This endpoint only provides the following content types: {}'.format(', '.join(
                     self.response_renderers.keys())
                 )
             )
-        else:
-            accept_type = request.accept_mimetypes.best
+
         renderer = self.response_renderers[accept_type]
 
         response = method(*args, **kwargs)
@@ -135,7 +169,9 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
         parent_filter = self._get_parent_filter(request.url, kwargs)
         objects_count, objects = self.get_collection(qs, kwargs, filters=parent_filter)
 
-        schema_kwargs = getattr(self, 'get_schema_kwargs', dict())
+        # get_schema_kwargs can be a class variable or a function
+        schema_kwargs = self._access_kwargs('get_schema_kwargs', args, kwargs)
+        schema_kwargs.update()
         schema_kwargs.update({'many': True})
 
         self.before_marshmallow(args, kwargs)
@@ -151,7 +187,7 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
         add_pagination_links(result,
                              objects_count,
                              qs,
-                             url_for(self.view, _external=True, **view_kwargs))
+                             url_for(request.endpoint, _external=True, **view_kwargs))
 
         result.update({'meta': {'count': objects_count}})
 
@@ -165,8 +201,9 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
         qs = QSManager(request.args, self.schema)
         json_data = self.parse_request()
 
+        schema_kwargs = self._access_kwargs('post_schema_kwargs', args, kwargs)
         schema = compute_schema(self.schema,
-                                getattr(self, 'post_schema_kwargs', dict()),
+                                schema_kwargs,
                                 qs,
                                 qs.include)
 
@@ -257,8 +294,9 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
 
         self.before_marshmallow(args, kwargs)
 
+        schema_kwargs = self._access_kwargs('get_schema_kwargs', args, kwargs)
         schema = compute_schema(self.schema,
-                                getattr(self, 'get_schema_kwargs', dict()),
+                                schema_kwargs,
                                 qs,
                                 qs.include)
 
@@ -273,7 +311,7 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
         """Update an object"""
         qs = QSManager(request.args, self.schema)
         json_data = self.parse_request()
-        schema_kwargs = getattr(self, 'patch_schema_kwargs', dict())
+        schema_kwargs = self._access_kwargs('patch_schema_kwargs', args, kwargs)
         schema_kwargs.update({'partial': True})
 
         self.before_marshmallow(args, kwargs)
